@@ -2,15 +2,16 @@ package Term::Choose::LineFold;
 
 use warnings;
 use strict;
-use 5.008003;
+use 5.10.0;
 
-our $VERSION = '1.713';
+our $VERSION = '1.745';
 
 use Exporter qw( import );
 
 our @EXPORT_OK = qw( line_fold print_columns cut_to_printwidth );
 
-use Term::Choose::Screen qw( normal );
+use Term::Choose::Constants qw( WIDTH_CURSOR );
+use Term::Choose::Screen    qw( normal );
 
 
 BEGIN {
@@ -69,7 +70,7 @@ sub print_columns {
 
 
 sub cut_to_printwidth {
-    my ( $str, $avail_w, $return_rest ) = @_;
+    my ( $str, $avail_width, $return_remainder ) = @_;
     my $count = 0;
     my $total = 0;
     for my $i ( 0 .. ( length( $str ) - 1 ) ) {
@@ -77,32 +78,34 @@ sub cut_to_printwidth {
         if ( ! defined $cache->[$c] ) {
             $cache->[$c] = char_width( $c )
         }
-        if ( ( $total = $total + $cache->[$c] ) > $avail_w ) {
-            if ( ( $total - $cache->[$c] ) < $avail_w ) {
-                return substr( $str, 0, $count ) . ' ', substr( $str, $count ) if $return_rest;
+        if ( ( $total = $total + $cache->[$c] ) > $avail_width ) {
+            if ( ( $total - $cache->[$c] ) < $avail_width ) {
+                return substr( $str, 0, $count ) . ' ', substr( $str, $count ) if $return_remainder;
                 return substr( $str, 0, $count ) . ' ';
             }
-            return substr( $str, 0, $count ), substr( $str, $count ) if $return_rest;
+            return substr( $str, 0, $count ), substr( $str, $count ) if $return_remainder;
             return substr( $str, 0, $count );
 
         }
         ++$count;
     }
-    return $str, '' if $return_rest;
+    return $str, '' if $return_remainder;
     return $str;
 }
 
 
 sub line_fold {
-    my ( $str, $avail_w, $opt ) = @_; #copy $str
-    return $str if ! defined $str || ! length $str;
+    my ( $str, $avail_width, $opt ) = @_; #copy $str
+    if ( ! defined $str || ! length $str ) {
+        return $str;
+    }
     for ( $opt->{init_tab}, $opt->{subseq_tab} ) {
         if ( defined $_ && length $_ ) {
             s/\t/ /g;
-            s/[\x{000a}-\x{000d}\x{0085}\x{2028}\x{2029}]+/\ \ /g;
+            s/\v+/\ \ /g;
             s/[\p{Cc}\p{Noncharacter_Code_Point}\p{Cs}]//g;
-            if ( length > $avail_w / 4 ) {
-                $_ = cut_to_printwidth( $_, int( $avail_w / 2 ) );
+            if ( length > $avail_width / 4 ) {
+                $_ = cut_to_printwidth( $_, int( $avail_width / 2 ) );
             }
         }
         else {
@@ -115,22 +118,21 @@ sub line_fold {
         $str =~ s/(\e\[[\d;]*m)/push( @color, $1 ) && "\x{feff}"/ge;
     }
     $str =~ s/\t/ /g;
-    $str =~ s/[^\x{0a}\x{0b}\x{0c}\x{0d}\x{85}\P{Cc}]//g; # remove control chars but keep vertical spaces
+    $str =~ s/[^\v\P{Cc}]//g; # remove control chars but keep vertical spaces
     $str =~ s/[\p{Noncharacter_Code_Point}\p{Cs}]//g;
-    my $regex = qr/\x{0d}\x{0a}|[\x{000a}-\x{000d}\x{0085}\x{2028}\x{2029}]/; # \R 5.10
-    if ( $str !~ /$regex/ && print_columns( $opt->{init_tab} . $str ) <= $avail_w && ! @color ) {
+    if ( $str !~ /\R/ && print_columns( $opt->{init_tab} . $str ) <= $avail_width && ! @color ) {
         return $opt->{init_tab} . $str;
     }
     my @paragraphs;
 
-    for my $row ( split /$regex/, $str, -1 ) { # -1 to keep trailing empty fields
+    for my $row ( split /\R/, $str, -1 ) { # -1 to keep trailing empty fields
         my @lines;
         $row =~ s/\s+\z//;
         my @words = split( /(?<=\S)(?=\s)/, $row );
         my $line = $opt->{init_tab};
 
         for my $i ( 0 .. $#words ) {
-            if ( print_columns( $line . $words[$i] ) <= $avail_w ) {
+            if ( print_columns( $line . $words[$i] ) <= $avail_width ) {
                 $line .= $words[$i];
             }
             else {
@@ -143,11 +145,11 @@ sub line_fold {
                     $words[$i] =~ s/^\s+//;
                     $tmp = $opt->{subseq_tab} . $words[$i];
                 }
-                ( $line, my $remainder ) = cut_to_printwidth( $tmp, $avail_w, 1 );
+                ( $line, my $remainder ) = cut_to_printwidth( $tmp, $avail_width, 1 );
                 while ( length $remainder ) {
                     push( @lines, $line );
                     $tmp = $opt->{subseq_tab} . $remainder;
-                    ( $line, $remainder ) = cut_to_printwidth( $tmp, $avail_w, 1 );
+                    ( $line, $remainder ) = cut_to_printwidth( $tmp, $avail_width, 1 );
                 }
             }
             if ( $i == $#words ) {
