@@ -2,23 +2,47 @@ package Term::Choose::Win32;
 
 use warnings;
 use strict;
-use 5.008003;
+use 5.10.0;
 
-our $VERSION = '1.713';
+our $VERSION = '1.745';
 
 
 use Encode qw( decode );
 
 use Encode::Locale qw();
-use Win32::Console qw( STD_INPUT_HANDLE ENABLE_MOUSE_INPUT ENABLE_PROCESSED_INPUT STD_OUTPUT_HANDLE
+use Win32::Console qw( STD_INPUT_HANDLE ENABLE_PROCESSED_INPUT ENABLE_LINE_INPUT ENABLE_ECHO_INPUT ENABLE_MOUSE_INPUT
+                       STD_OUTPUT_HANDLE
                        RIGHT_ALT_PRESSED LEFT_ALT_PRESSED RIGHT_CTRL_PRESSED LEFT_CTRL_PRESSED SHIFT_PRESSED
                        FOREGROUND_INTENSITY BACKGROUND_INTENSITY );
 
 use Win32::Console::PatchForRT33513 qw();
-use Term::Choose::Constants         qw( :win32 :keys );
+use Term::Choose::Constants         qw( :all );
 use Term::Choose::Screen            qw( hide_cursor show_cursor normal );
 
 
+use constant {
+    MOUSE_WHEELED                => 0x0004,
+    LEFTMOST_BUTTON_PRESSED      => 0x0001,
+    RIGHTMOST_BUTTON_PRESSED     => 0x0002,
+    FROM_LEFT_2ND_BUTTON_PRESSED => 0x0004,
+
+    VK_CODE_PAGE_UP   => 33,
+    VK_CODE_PAGE_DOWN => 34,
+    VK_CODE_END       => 35,
+    VK_CODE_HOME      => 36,
+    VK_CODE_LEFT      => 37,
+    VK_CODE_UP        => 38,
+    VK_CODE_RIGHT     => 39,
+    VK_CODE_DOWN      => 40,
+    VK_CODE_INSERT    => 45,
+    VK_CODE_DELETE    => 46,
+    VK_CODE_F1        => 112,
+    VK_CODE_F2        => 113,
+    VK_CODE_F3        => 114,
+    VK_CODE_F4        => 115,
+};
+
+
 sub SHIFTED_MASK () {
       RIGHT_ALT_PRESSED
     | LEFT_ALT_PRESSED
@@ -63,6 +87,10 @@ sub __get_key_OS {
             elsif ( $v_key_code == VK_CODE_DOWN )      { return VK_DOWN }
             elsif ( $v_key_code == VK_CODE_INSERT )    { return VK_INSERT }
             elsif ( $v_key_code == VK_CODE_DELETE )    { return VK_DELETE }
+            elsif ( $v_key_code == VK_CODE_F1 )        { return VK_F1 }
+            elsif ( $v_key_code == VK_CODE_F2 )        { return VK_F2 }
+            elsif ( $v_key_code == VK_CODE_F3 )        { return VK_F3 }
+            elsif ( $v_key_code == VK_CODE_F4 )        { return VK_F4 }
             else                                       { return NEXT_get_key }
         }
     }
@@ -99,16 +127,18 @@ sub __get_key_OS {
 
 sub __set_mode {
     my ( $self, $config ) = @_;
-    $self->{hide_cursor} = $config->{hide_cursor};
     $self->{input} = Win32::Console->new( STD_INPUT_HANDLE );
     $self->{old_in_mode} = $self->{input}->Mode();
     if ( $config->{mouse} ) {
-        $self->{input}->Mode( !ENABLE_PROCESSED_INPUT|ENABLE_MOUSE_INPUT );
+        # To make the mouse mode work, QUICK_EDIT_MODE has to be disabled.
+        # https://docs.microsoft.com/en-us/windows/console/setconsolemode:
+        # To disable this mode (QUICK_EDIT_MODE), use ENABLE_EXTENDED_FLAGS (0x0080) without this flag (QUICK_EDIT_MODE).
+        $self->{input}->Mode( !ENABLE_PROCESSED_INPUT|ENABLE_MOUSE_INPUT|0x0080 );
     }
     else {
         $self->{input}->Mode( !ENABLE_PROCESSED_INPUT );
     }
-    if ( $self->{hide_cursor} ) {
+    if ( $config->{hide_cursor} ) {
         print hide_cursor();
     }
     return $config->{mouse};
@@ -116,16 +146,26 @@ sub __set_mode {
 
 
 sub __reset_mode {
-    my ( $self ) = @_;
+    my ( $self, $config ) = @_;
+    my $fallback_resetmode = ENABLE_PROCESSED_INPUT|ENABLE_LINE_INPUT|ENABLE_ECHO_INPUT;
     if ( defined $self->{input} ) {
         if ( defined $self->{old_in_mode} ) {
             $self->{input}->Mode( $self->{old_in_mode} );
+            # old_in_mode == 503 == 0x0001|0x0002|0x0004|0x0010|0x0020|0x0040|0x0080|0x0100
             delete $self->{old_in_mode};
         }
+        else {
+            $self->{input}->Mode( $fallback_resetmode );
+        }
         $self->{input}->Flush;
     }
+    else {
+        my $input = Win32::Console->new( STD_INPUT_HANDLE );
+        $input->Mode( $fallback_resetmode );
+        $input->Flush;
+    }
     print normal();
-    if ( delete $self->{hide_cursor} ) {
+    if ( $config->{hide_cursor} ) {
         print show_cursor();
     }
 }
@@ -146,6 +186,16 @@ sub __get_cursor_row {
 
 
 
+# ENABLE_PROCESSED_INPUT        = 0x0001    win32::Console
+# ENABLE_LINE_INPUT             = 0x0002    Win32::Console
+# ENABLE_ECHO_INPUT             = 0x0004    Win32::Console
+# ENABLE_WINDOW_INPUT           = 0x0008    Win32::Console
+# ENABLE_MOUSE_INPUT            = 0x0010    Win32::Console
+# ENABLE_INSERT_MODE            = 0x0020
+# ENABLE_QUICK_EDIT_MODE        = 0x0040
+# ENABLE_EXTENDED_FLAGS         = 0x0080
+# ENABLE_AUTO_POSITION          = 0x0100    ?
+# ENABLE_VIRTUAL_TERMINAL_INPUT = 0x0200
 
 
 
